@@ -1,15 +1,13 @@
 {-# LANGUAGE CPP #-}
-#ifdef __GLASGOW_HASKELL__
 #if __GLASGOW_HASKELL__ >= 707
-{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving, Safe #-}
+{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving, Safe, DefaultSignatures #-}
 #elif __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy #-}
-#endif
+{-# LANGUAGE Trustworthy, DefaultSignatures #-}
 #endif
  -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Comonad
--- Copyright   :  (C) 2008-2012 Edward Kmett,
+-- Copyright   :  (C) 2008-2015 Edward Kmett,
 --                (C) 2004 Dave Menendez
 -- License     :  BSD-style (see the file LICENSE)
 --
@@ -24,6 +22,7 @@ module Control.Comonad (
   , liftW     -- :: Comonad w => (a -> b) -> w a -> w b
   , wfix      -- :: Comonad w => w (w a -> a) -> a
   , cfix      -- :: Comonad w => (w a -> a) -> w a
+  , kfix      -- :: ComonadApply w => w (w a -> a) -> w a
   , (=>=)
   , (=<=)
   , (<<=)
@@ -42,6 +41,7 @@ module Control.Comonad (
   ) where
 
 -- import _everything_
+import Data.Functor
 import Control.Applicative
 import Control.Arrow
 import Control.Category
@@ -56,12 +56,15 @@ import Data.Functor.Identity
 import Data.List.NonEmpty hiding (map)
 import Data.Semigroup hiding (Product)
 import Data.Tagged
-import Data.Tree
 import Prelude hiding (id, (.))
 import Control.Monad.Fix
 import Data.Typeable
 
-infixl 4 <@, @>, <@@>, <@>, $>
+#ifdef MIN_VERSION_containers
+import Data.Tree
+#endif
+
+infixl 4 <@, @>, <@@>, <@>
 infixl 1 =>>
 infixr 1 <<=, =<=, =>=
 
@@ -139,6 +142,10 @@ class Functor w => Comonad w where
   extend :: (w a -> b) -> w a -> w b
   extend f = fmap f . duplicate
 
+#if __GLASGOW_HASKELL__ >= 708
+  {-# MINIMAL extract, (duplicate | extend) #-}
+#endif
+
 
 instance Comonad ((,)e) where
   duplicate p = (fst p, p)
@@ -146,6 +153,16 @@ instance Comonad ((,)e) where
   extract = snd
   {-# INLINE extract #-}
 
+#if MIN_VERSION_semigroups(0,16,2)
+instance Comonad (Arg e) where
+  duplicate w@(Arg a _) = Arg a w
+  {-# INLINE duplicate #-}
+  extend f w@(Arg a _) = Arg a (f w)
+  {-# INLINE extend #-}
+  extract (Arg _ b) = b
+  {-# INLINE extract #-}
+#endif
+
 instance Monoid m => Comonad ((->)m) where
   duplicate f m = f . mappend m
   {-# INLINE duplicate #-}
@@ -169,10 +186,12 @@ instance Comonad w => Comonad (IdentityT w) where
   extract = extract . runIdentityT
   {-# INLINE extract #-}
 
+#ifdef MIN_VERSION_containers
 instance Comonad Tree where
   duplicate w@(Node _ as) = Node w (map duplicate as)
   extract (Node a _) = a
   {-# INLINE extract #-}
+#endif
 
 instance Comonad NonEmpty where
   extend f w@ ~(_ :| aas) = f w :| case aas of
@@ -212,6 +231,10 @@ instance Comonad NonEmpty where
 
 class Comonad w => ComonadApply w where
   (<@>) :: w (a -> b) -> w a -> w b
+#if __GLASGOW_HASKELL__ >= 702
+  default (<@>) :: Applicative w => w (a -> b) -> w a -> w b
+  (<@>) = (<*>)
+#endif
 
   (@>) :: w a -> w b -> w b
   a @> b = const id <$> a <@> b
@@ -240,10 +263,12 @@ instance ComonadApply Identity where
 instance ComonadApply w => ComonadApply (IdentityT w) where
   IdentityT wa <@> IdentityT wb = IdentityT (wa <@> wb)
 
+#ifdef MIN_VERSION_containers
 instance ComonadApply Tree where
   (<@>) = (<*>)
   (<@ ) = (<* )
   ( @>) = ( *>)
+#endif
 
 -- | A suitable default definition for 'fmap' for a 'Comonad'.
 -- Promotes a function to a comonad.
@@ -259,15 +284,22 @@ liftW :: Comonad w => (a -> b) -> w a -> w b
 liftW f = extend (f . extract)
 {-# INLINE liftW #-}
 
--- | Comonadic fixed point à la Menendez
+-- | Comonadic fixed point à la David Menendez
 wfix :: Comonad w => w (w a -> a) -> a
 wfix w = extract w (extend wfix w)
 
--- | Comonadic fixed point à la Orchard
+-- | Comonadic fixed point à la Dominic Orchard
 cfix :: Comonad w => (w a -> a) -> w a
 cfix f = fix (extend f)
 {-# INLINE cfix #-}
 
+-- | Comonadic fixed point à la Kenneth Foner:
+--
+-- This is the @evaluate@ function from his <https://www.youtube.com/watch?v=F7F-BzOB670 "Getting a Quick Fix on Comonads"> talk.
+kfix :: ComonadApply w => w (w a -> a) -> w a
+kfix w = fix $ \u -> w <@> duplicate u
+{-# INLINE kfix #-}
+
 -- | 'extend' with the arguments swapped. Dual to '>>=' for a 'Monad'.
 (=>>) :: Comonad w => w a -> (w a -> b) -> w b
 (=>>) = flip extend
@@ -356,6 +388,12 @@ instance Monad (Cokleisli w a) where
   return = Cokleisli . const
   Cokleisli k >>= f = Cokleisli $ \w -> runCokleisli (f (k w)) w
 
+#if !(MIN_VERSION_base(4,7,0))
+
+infixl 4 $>
+
 -- | Replace the contents of a functor uniformly with a constant value.
 ($>) :: Functor f => f a -> b -> f b
 ($>) = flip (<$)
+
+#endif
