@@ -1,14 +1,13 @@
 {-# LANGUAGE CPP #-}
 #if __GLASGOW_HASKELL__ >= 707
-{-# LANGUAGE StandaloneDeriving, DeriveDataTypeable #-}
-#endif
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+{-# LANGUAGE StandaloneDeriving, DeriveDataTypeable, Safe #-}
+#elif __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Comonad.Trans.Traced
--- Copyright   :  (C) 2008-2013 Edward Kmett
+-- Copyright   :  (C) 2008-2014 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 --
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
@@ -37,19 +36,32 @@ module Control.Comonad.Trans.Traced
   , censor
   ) where
 
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ < 707
+#endif
+
+#if __GLASGOW_HASKELL__ < 707
 import Control.Monad.Instances ()
 #endif
+
 import Control.Monad (ap)
 import Control.Comonad
 import Control.Comonad.Hoist.Class
 import Control.Comonad.Trans.Class
+
+#ifdef MIN_VERSION_distributive
 import Data.Distributive
+#endif
+
 import Data.Functor.Identity
+
+#if __GLASGOW_HASKELL__ < 710
 import Data.Semigroup
+#endif
+
 import Data.Typeable
 
+
 type Traced m = TracedT m Identity
 
 traced :: (m -> a) -> Traced m a
@@ -80,8 +92,10 @@ instance Monoid m => ComonadTrans (TracedT m) where
 instance Monoid m => ComonadHoist (TracedT m) where
   cohoist l = TracedT . l . runTracedT
 
+#ifdef MIN_VERSION_distributive
 instance Distributive w => Distributive (TracedT m w) where
   distribute = TracedT . fmap (\tma m -> fmap ($ m) tma) . collect runTracedT
+#endif
 
 trace :: Comonad w => m -> TracedT m w a -> a
 trace m (TracedT wf) = extract wf m
